from dios import *

v0 = 'var0'
v1 = 'var1'
v2 = 'var2'
v3 = 'var3'
v4 = 'var4'
v5 = 'var5'
v6 = 'var6'
v7 = 'var7'
v8 = 'var8'
v9 = 'var9'


def gen_series(rows, randomize=True):
    start = dt.datetime.strptime("2000-01-10 00:00:00", "%Y-%m-%d %H:%M:%S")
    if randomize:
        sec = np.random.randint(1, 10)
        rows = np.random.randint(int(rows * 0.5), rows * 2)
        start += pd.Timedelta(sec)
    else:
        sec = 10
    times = pd.date_range(periods=rows, start=start, freq=f'{sec}s')
    d = np.random.randint(1, 9, rows)
    return pd.Series(data=d, index=times)


def test_getitem():
    # prepare
    begin = dt.datetime.strptime("2000-01-10 00:00:00", "%Y-%m-%d %H:%M:%S")
    t0 = begin + pd.Timedelta('20s')
    t1 = t0 + pd.Timedelta('50s')
    dos_aligned = DictOfSeries()
    dos_rand = DictOfSeries()

    # fill
    vars = 10
    rows = 10
    for i in range(0, vars):
        dos_aligned[f'var{i}'] = gen_series(rows, randomize=False)
        dos_rand[f'var{i}'] = gen_series(rows, randomize=True)

    # testsets
    var = [v0, [], [v1], [v0, v0], [v0, v2]]
    tssl = [slice(None), slice(t0, None), slice(None, t1), slice(t0, t1), slice(t0, t0)]
    ts = [t0, [], [t0], [t0, t0], [t0, t1]]

    for v in var:
        print(v)
        dos_rand[v]

    # use aligned dios for time stamps instead of time-ranges
    for t in tssl:
        print(t)
        dos_rand[v]

    try:
        dos_aligned[v1, v2]
    except KeyError:
        pass

    try:
        dos_aligned[v1, v2, v3]
    except KeyError:
        pass


def test_setitem():
    # prepare
    begin = dt.datetime.strptime("2000-01-10 00:00:00", "%Y-%m-%d %H:%M:%S")
    t0 = begin + pd.Timedelta('30s')
    t1 = t0 + pd.Timedelta('50s')
    dos_aligned = DictOfSeries()
    dos_aligned.name = 'aligned'
    dos_rand = DictOfSeries()
    dos_rand.name = 'rand'

    # fill
    vars = 10
    rows = 100
    for i in range(0, vars):
        dos_aligned[f'var{i}'] = gen_series(rows, randomize=False)
        dos_rand[f'var{i}'] = gen_series(rows, randomize=True)

    # testsets
    keys = [v0, [v1], [v0, v0], [v0, v2]]
    tssl = [slice(None), slice(t0, None), slice(None, t1), slice(t0, t1), slice(t0, t0)]
    scalars = [1, 'deadbeef']
    l = list(np.random.randint(0, 100, rows))
    dos = [dos_aligned.copy(), dos_rand.copy()]
    for d in dos:
        d.name = 'src-dios'

    # assign scalars
    for val in scalars:
        for v in keys:
            print(v, '=', val)
            dos_rand[v] = val

    # assign scalars
    for val in scalars:
        for t in tssl:
            print(t, '=', val)
            dos_rand[t] = val

    # assign list
    for v in keys:
        print(v, '=', l)
        dos_aligned[v] = l

    # assign series
    for v in keys:
        print(v, '=', 'series')
        dos_aligned[v] = dos_aligned[v4]

    # assign dios
    for v in keys:
        for d in dos:
            print(f'{v} = dios[{v}]')
            dos_aligned[v] = d[v]

    # probkeys = [[], slice(v0, v0), ]
    # for v in probkeys:
    #     try:
    #         dos_aligned[v] = l
    #     except ValueError:
    #         pass
    #     else:
    #         raise AssertionError("should return an error")


def test_integrity():
    rows = 1000
    cols = 10
    df, _, _, dos = get_testset(1000, 10, storagedir='../dios/profiling/testsets')

    v = var_prefix + str(np.random.randint(0, cols))
    t = find_index_range(dos)
    t0, t1 = gen_random_timestamps(*t)

    # originals
    dos_ = dos.copy(deep=False)
    df_ = df.copy()
    s_ = df[v].dropna()

    # identity
    assert dos_ == dos
    assert (dos_[v] == dos[v]).all()
    assert dos_ is not dos
    assert dos_[v] is not dos[v]

    # equal t0 df
    assert dos[v] is not df[v]
    assert (dos[v] == df[v].dropna()).all()

    # write
    dos = dos_.copy()
    s = s_.copy()
    dos[t0:t1] = 4
    s[t0:t1] = 4
    assert (dos[v] == s).all()

    # write all
    s = s_.copy()
    dos = dos_.copy()
    dos[v] = 111
    s[:] = 111
    assert (dos[v] == s).all()

    # multi variables - slice
    df = df_.copy()
    dos = dos_.copy()
    li = [v0, v1, v2]
    dos[t0:t1] = 222
    for x in li:
        s = df[x].dropna()
        s[t0:t1] = 222
        assert (dos[x] == s).all()
    # on all
    dos[t0:t1] = 222.111
    m = df.loc[t0:t1,:].notna()
    df[m] = 222.111
    for x in df:
        s = df[x].dropna()
        assert (dos[x] == s).all()

    # multi variables - list
    df = df_.copy()
    dos = dos_.copy()
    li = [v0, v5, v3, v9]
    dos[t0:t1] = 333
    for x in li:
        s = df[x].dropna()
        s[t0:t1] = 333
        assert (dos[x] == s).all()

    # dos to dos
    df = df_.copy()
    dos = dos_.copy()
    dos[v] = 444
    dos[v5] = dos[v] * 0.1
    s = df[v].dropna()
    s[:] = 444 * 0.1
    assert (dos[v5] == s).all()


def test_foreach():
    # dios.foreach() is alias for dios.pipe()

    dios = DictOfSeries()
    ser = pd.Series([1,4,5,6,3,5,5,3,2,1,13])
    dios['a'] = ser
    dios['b'] = ser * 4

    # return type of function: Series
    d = dios.pipe(pd.Series.astype, float)
    assert isinstance(d, DictOfSeries)
    assert not d['a'].dtype == ser.dtype
    assert (d['a'] == ser.astype(float)).all()

    # return type of function: scalar
    d = dios.pipe(min)
    assert isinstance(d, pd.Series)
    assert d['a'] == ser.min()
    assert d['b'] == ser.min() * 4

    # return type of function: scalar
    # but with squeeze
    d = dios.copy()
    del d['a']
    d = d.pipe(min)
    assert isinstance(d, np.int64)
    assert d == 4

    # test inplace
    d = dios.copy()
    d1 = dios.copy()
    d2 = d.pipe(pd.Series.drop_duplicates, inplace=False)
    # original didtn change
    assert d is not d1 and d == d1
    # tests on d2
    assert d2 is not d
    assert isinstance(d2, DictOfSeries)
    assert (d2['a'] == ser.copy().drop_duplicates()).all()
    assert (d2['b'] == (ser * 4 ).copy().drop_duplicates()).all()

    d3 = d.pipe(pd.Series.drop_duplicates, inplace=True)
    assert d3 is None
    # original did (!) change
    assert d is not d1
    try: d == d1
    except ValueError: pass;
    else: raise AssertionError
    # check if gone well..
    assert (d['a'] == ser.copy().drop_duplicates()).all()
    assert (d['b'] == (ser * 4 ).copy().drop_duplicates()).all()


def tmptest():
    # prepare
    begin = dt.datetime.strptime("2000-01-10 00:00:00", "%Y-%m-%d %H:%M:%S")
    t0 = begin + pd.Timedelta('20s')
    t1 = t0 + pd.Timedelta('50s')
    dosa = DictOfSeries()
    dosr = DictOfSeries()

    # fill
    vars = 10
    rows = 8
    for i in range(0, vars):
        dosa[f'v{i}'] = gen_series(rows, randomize=False)
        dosr[f'v{i}'] = gen_series(rows, randomize=True)

    dosa[[], slice(None)] = 99
    exit(9)

    print(dosa)
    d = dosa[t0:t1, v1]
    d = d * 1000
    print(d)
    print()
    print(dosa[t0:t1, v1])
    l1 = len(dosa[t0:t1, v1])
    l2 = len(d)
    print(l1, l2)
    dosa[t0:t1, v1] = d
    print(dosa[v1])
    exit(3)


if __name__ == '__main__':
    dios_options['disp_max_rows'] = 20

    # tmptest()
    test_getitem()
    test_setitem()
    test_integrity()
    test_foreach()
